/*
 * gsampler-console.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 *
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <glib/gi18n-lib.h>

#include "gsampler-console.h"
#include "gsampler-client.h"
#include "gsampler-window.h"

G_DEFINE_TYPE (GsamplerConsole, gsampler_console, GTK_TYPE_NOTEBOOK);

static void
gsampler_console_write_buffer (GsamplerConsole* self, const gchar* message, gboolean ret)
{
	GtkTextIter iter = {0};
	GtkTextMark* mark = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (message != NULL);

	mark = gtk_text_buffer_get_mark (self->priv->buffer, "base");
	gtk_text_buffer_get_iter_at_mark (self->priv->buffer, &iter, mark);

	if (ret) 
	{
		gtk_text_buffer_insert_with_tags_by_name (self->priv->buffer, &iter, message,
		                                          strlen (message), "red", NULL, NULL);
	} else 
	{
		gtk_text_buffer_insert (self->priv->buffer, &iter, message, strlen (message));
	}
	gtk_text_buffer_get_end_iter (self->priv->buffer, &iter);
	gtk_text_buffer_move_mark (self->priv->buffer, mark, &iter);
	if (self->priv->eventoutput != NULL &&
	    gtk_widget_get_visible ((GtkWidget*) self->priv->eventoutput) == TRUE) {
		gtk_text_view_scroll_to_mark (self->priv->eventoutput, mark, (gdouble) 0,
		                              FALSE, (gdouble) 0, (gdouble) 0);
	}
}

static gboolean
gsampler_console_write_buffer_thread (gpointer data)
{
	GsamplerConsole* self = (GsamplerConsole*) data;
	g_return_val_if_fail (GSAMPLER_IS_CONSOLE (self), FALSE);

	//gdk_threads_enter ();
	if (self->priv->console_mes != NULL && self->priv->console_mes->len != 0)
	{
		gchar *mes = g_string_free (self->priv->console_mes, FALSE);
		gsampler_console_write_buffer (self, mes, FALSE);
		g_free (mes);
		self->priv->console_mes = g_string_new ("");
	}
	//gdk_threads_leave ();
	return FALSE;
}

static void
gsampler_console_write_server_buffer (GsamplerConsole* self, const gchar* message)
{
	GtkTextIter iter = {0};
	GtkTextMark* mark = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (message != NULL);

	gtk_text_buffer_get_end_iter (self->priv->server_buffer, &iter);
	gtk_text_buffer_insert (self->priv->server_buffer, &iter, message, strlen (message));
	mark = gtk_text_buffer_get_insert (self->priv->server_buffer);
	gtk_text_view_scroll_to_mark (self->priv->server_output, mark, (gdouble) 0,
		                              FALSE, (gdouble) 0, (gdouble) 0);
}

static void
gsampler_console_on_entry_activated (GtkEntry* sender, gpointer data)
{
	GsamplerClient* client;
	gchar* com;
	GsamplerConsole* self = (GsamplerConsole*) data;
	g_return_if_fail (GSAMPLER_IS_CONSOLE (self));
	g_return_if_fail (sender != NULL);

	client = gsampler_client_get_instance ();
	com = g_strstrip (g_strdup ((gchar*) gtk_entry_get_text (sender)));
	if (com != NULL) {
		gchar* ret;
		ret = gsampler_client_send_commande (client, com);
		ret = g_strconcat (ret, "\n", NULL);
		gsampler_console_write_buffer (self, ret, TRUE);
		g_free (ret);
	}
	g_free (com);
	g_object_unref (client);
}

static GObject *
gsampler_console_constructor (GType type, guint n_construct_properties,
                               GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GsamplerConsole * self = NULL;
	GtkTextIter iter = {0};
	GtkTextMark* mark;
	GtkScrolledWindow* scroll1;
	GtkScrolledWindow* scroll2;
	GtkBox* vbox;
	GtkBox* hbox;
	GtkWidget* send_lbl;
	GtkWidget* cons_lbl;
	GtkWidget* serv_lbl;
	
	obj = G_OBJECT_CLASS (gsampler_console_parent_class)->constructor (type, n_construct_properties, construct_properties);
	self = GSAMPLER_CONSOLE (obj);
	g_object_set (self, "height-request", 200, NULL);
	gtk_container_set_border_width ((GtkContainer*) self, 2);
	
	self->priv->buffer = gtk_text_buffer_new (NULL);
	gtk_text_buffer_create_tag (self->priv->buffer, "red", "foreground", "red", NULL, NULL);
	mark = gtk_text_mark_new ("base", FALSE);
	gtk_text_buffer_get_start_iter (self->priv->buffer, &iter);
	gtk_text_buffer_add_mark (self->priv->buffer, mark, &iter);
	g_object_unref (mark);
	self->priv->eventoutput = (GtkTextView*) gtk_text_view_new_with_buffer (self->priv->buffer);
	vbox = (GtkBox*) gtk_vbox_new (FALSE, 0);
	scroll1 = (GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (scroll1, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_text_view_set_cursor_visible (self->priv->eventoutput, FALSE);
	gtk_text_view_set_editable (self->priv->eventoutput, FALSE);
	gtk_text_view_set_wrap_mode (self->priv->eventoutput, GTK_WRAP_WORD);
	gtk_container_add ((GtkContainer*) scroll1, (GtkWidget*) self->priv->eventoutput);
	hbox = (GtkBox*) gtk_hbox_new (FALSE, 0);
	self->priv->send_com = gtk_entry_new ();
	send_lbl = gtk_label_new (_("Command : "));
	gtk_box_pack_start (hbox, send_lbl, FALSE, TRUE, 5);
	gtk_box_pack_start (hbox, self->priv->send_com, TRUE, TRUE, 5);
	gtk_box_pack_start (vbox, (GtkWidget*) scroll1, TRUE, TRUE, 5);
	gtk_box_pack_start (vbox, (GtkWidget*) hbox, FALSE, TRUE, 5);
	cons_lbl = gtk_label_new (_("Console"));
	gtk_notebook_append_page ((GtkNotebook*) self, (GtkWidget*) vbox, cons_lbl);
	
	self->priv->server_buffer = gtk_text_buffer_new (NULL);
	self->priv->server_output = (GtkTextView*) gtk_text_view_new_with_buffer (self->priv->server_buffer);
	scroll2 = (GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (scroll2, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_text_view_set_cursor_visible (self->priv->server_output, FALSE);
	gtk_text_view_set_editable (self->priv->server_output, FALSE);
	gtk_text_view_set_wrap_mode (self->priv->server_output, GTK_WRAP_WORD);
	gtk_container_add ((GtkContainer*) scroll2, (GtkWidget*) self->priv->server_output);
	serv_lbl = gtk_label_new (_("Server"));
	gtk_notebook_append_page ((GtkNotebook*) self, (GtkWidget*) scroll2, serv_lbl);

	gtk_widget_set_sensitive ((GtkWidget*) self->priv->send_com, FALSE);
	self->priv->console_mes = g_string_new ("");
	return obj;
}

static void
gsampler_console_init (GsamplerConsole * self)
{
	self->priv = GSAMPLER_CONSOLE_GET_PRIVATE (self);
	
}

static void
gsampler_console_finalize (GObject* obj)
{
	GsamplerConsole * self;
	self = GSAMPLER_CONSOLE (obj);
	g_string_free (self->priv->console_mes, TRUE);
	G_OBJECT_CLASS (gsampler_console_parent_class)->finalize (obj);
}

static void
gsampler_console_class_init (GsamplerConsoleClass * klass)
{
	gsampler_console_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GsamplerConsolePrivate));
	G_OBJECT_CLASS (klass)->finalize = gsampler_console_finalize;
	G_OBJECT_CLASS (klass)->constructor = gsampler_console_constructor;
}

GsamplerConsole*
gsampler_console_new (void)
{
	GsamplerConsole * console = NULL;
	console = (GsamplerConsole*) g_object_new (GSAMPLER_TYPE_CONSOLE, NULL);
	return console;
}

void
gsampler_console_push_event (GsamplerConsole* self, const gchar* message, gpointer data)
{
	g_return_if_fail (self != NULL);
	g_return_if_fail (message != NULL);

	/*self->priv->console_mes = */g_string_append (self->priv->console_mes, message);
	gdk_threads_add_idle_full (G_PRIORITY_HIGH_IDLE, gsampler_console_write_buffer_thread,
	                 		   g_object_ref (self), g_object_unref);
}

void
gsampler_console_push_server_event (GsamplerConsole* self, const gchar* message, gpointer data)
{
	g_return_if_fail (self != NULL);
	g_return_if_fail (message != NULL);

	gsampler_console_write_server_buffer (self, message);
}

void
gsampler_console_active (GsamplerConsole* self)
{
	g_return_if_fail (self != NULL);

	gtk_widget_set_sensitive ((GtkWidget*) self->priv->send_com, TRUE);
	self->priv->send_com_id = g_signal_connect_object (self->priv->send_com, "activate",
	    	                 			(GCallback) gsampler_console_on_entry_activated, self, 0);
}

void
gsampler_console_inactive (GsamplerConsole* self)
{
	g_return_if_fail (self != NULL);

	gtk_widget_set_sensitive ((GtkWidget*) self->priv->send_com, FALSE);
	g_signal_handler_disconnect (self->priv->send_com, self->priv->send_com_id);
}
